'''Data loader module. (taken from pyweek skellington and extended)

Data must be in the data directory of your game.
The following filetypes are expected in subfolders in 'data/'
- Music: bgm
- Sounds: sfx
- Graphics: gfx

Mixer (sound) and display must be initialized to be loaded.
'''


import os
import pickle

import pygame

#path to data dir
__data_py__ = os.path.abspath(os.path.dirname(__file__))
__data_dir__ = os.path.normpath(os.path.join(__data_py__,'..','data'))


# taken from pyweek skellington
def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(__data_dir__, filename)

def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(os.path.join(__data_dir__, filename), mode)

def load_image(filename, colorkey=None, alpha=False):
    """Loads image from filename
    optional colorkey for transparency
    set alpha=True if the image already has transparency
    """
    path = os.path.join(__data_dir__, "gfx", filename)
    img = pygame.image.load(path)
    if colorkey:
        img.set_colorkey(colorkey)

    if alpha:
        return img.convert_alpha()
    else:
        return img.convert()

class NoneSound:
    """
    Dummy sound
    """
    def play(self):
        pass

def load_sound(filename):
    """Loads and returns sound object.
    Returns dummy sound if something is wrong with audio or file
    not found.
    """
    path = os.path.join(__data_dir__, "sfx", filename)
    try:
        return pygame.mixer.Sound(path)
    except:
        print "Could not load sound: ", path
    return NoneSound()

def load_gfx(filenames, colorkey=(0,255,0,255), alpha=False, folder=None):
    """receives: a list of filenames
    returns: dictionary with the loaded graphics

    The filenames are used as the dictionary's keys
    """
    dc_gfx = {}
    for name in filenames:
        path = name
        if folder:
            path = os.path.join(folder, name)
        dc_gfx[name] = load_image(path, colorkey, alpha)
        dc_gfx[name].set_colorkey(colorkey)
    return dc_gfx

def load_sfx(filenames):
    """receives a list of filenames
    returns: dictionary of sounds

    The filenames are used as the dictionary's keys
    """
    sounds = {}
    for name in filenames:
        sounds[name] = load_sound(name)
    return sounds

def load_object(filename):
    """
    unpickles and returns object from file
    """
    f = None
    try:
        f = open(filepath(filename),"rb")
        object = pickle.load(f)
        f.close()
        return object
    except:
        if f:
            f.close()
        print "Could not unpickle ", filename
        return None

def save_object(object, filename):
    """
    pickles object to file in 'data/' directory
    """
    f = None
    try:
        f = open(filepath(filename),"wb")
        pickle.dump(object, f, 2)
        f.close()
        return True
    except:
        if f:
            f.close()
        print "Could not pickle ", object, " to file ", filename
        return False

def load_grid(filename, width=32, height=32, colorkey=None, alpha=False):
    """Cuts image into several surfaces of (width, height) size.

    Receives: filename of image
    Returns: a nested list with the surfaces cut from the image

    Useful if you put a couple of graphics next to each other into one file
    (i.e. animations, tileset images, etc.).
    Note that these graphics must have the same size.

    Example:
    tileset = load_grid("snowland.png")
    tileset[1][2] would be the 2nd image in the 3rd row from that file
    """

    img = load_image(filename, alpha)
    sw, sh = img.get_size()

    if alpha:
        img.covert_alpha()
    else:
        img.convert()

    images = []
    surf = pygame.Surface((width, height))
    y = 0
    while y < sh:
        x = 0
        line = []
        while x < sw:
            if colorkey:
                surf = pygame.Surface((width, height))
                surf.set_colorkey(colorkey)
            #blit big image on smaller surface with x and y as offset
            surf.blit(img, (-x, -y))

            line.append(surf.copy())
            x += width
        images.append(line)
        y += height
    return images

class Music:
    """Simple music class.
    Keeps the music path and plays the music if
    """
    def __init__(self, filename):
        try:
            self.path = os.path.join(__data_dir__, "bgm", filename)
        except:
            print "Could not load music: ", path
            self.path = None

    def play(self, rep=0):
        """ play music
        """
        if self.path:
            pygame.mixer.music.stop()
            pygame.mixer.music.load(self.path)
            pygame.mixer.music.play(rep)

class JukeBox:
    """Handles music files, playback and stopping

    This is more like a basic music container than a jukebox.
    """
    def __init__(self, filenames):
        """receives: list of music filenames
        music paths are saved in a dictionary with the filenames as keys
        """
        self.bgm = {}
        for name in filenames:
            self.bgm[name] = Music(name)

    def play(self, filename, rep=0):
        """Play music, if filename is in Jukebox's dictionary
        """
        if filename in self.bgm.keys():
            self.bgm[filename].play(rep)

    def stop(self):
        """Stop music playback
        """
        pygame.mixer.music.stop()
