import sys
import pygame
from pygame.locals import *

from event import *

class Engine:
    """StateEgine
    Runs states, uses a list to stack states (when pausing a state)
    Attributes: states, current_state

    Example:
    e = Engine()  #initialise engine
    e.current_state = StartMenu()  #set starting state
    e.run()  #run your game

    That's basically all you need to know. Everything else will by done
    within the States.

    Confusing explanation of inner workings:
    Last state on state list is used as current_state.
    Mainloop of current_state is run.
    When current_state is done it is removed from the state list.
    If current_state is paused, it is kept on the state list.
    If current_state returns a state, this state is appended to
    state list to be run as next current_state.
    If the state list is empty the game will be closed.

    Note that you can't jump between the stacked states.
    The last state in the state list will ALWAYS be run as current_state.
    """
    def __init__(self):
        self.current_state = State() #default state
        self.states = [self.current_state]  #state handling list

    def run(self):
        """Engine starts, runs current_state
        """
        self.states = [self.current_state]

        while self.states:
            self.current_state = self.states.pop()
            if self.current_state.paused:
                self.current_state.unpause()

            next, paused = self.current_state.mainloop()
            if self.current_state.kill_prev and self.states:
                self.states.pop()  #kill previous state too, if requested..

            if paused:  #paused states are kept
                self.states.append(self.current_state)

            if next: #set next state
                self.states.append(next)


class State:
    """Basic State class.
    Attributes: fps, done, next, paused, kill_prev, widgets, events
    Returns: next_state, paused

    Important attributes explained:
    fps: frames per second (default: 20)
    next: next state (must be initialized, i.e. self.next_state=GameOver())
    kill_prev: if set to True, the previous (paused) state won't be resumed when this one quits
    widgets: append all buttons (or other widgets) to that list to have them updated
    """
    def __init__(self):
        self.done = False
        self.next = None
        self.clock = pygame.time.Clock()
        self.events = EventGroup()
        self.widgets = []
        self.paused = False
        self.kill_prev = False
        self.fps = 20

    def pause(self):
        """Should be called to pause state.
        Sets paused to True.
        """
        self.paused = True

    def unpause(self):
        """Called when Engine returns to this state
        """
        self.paused = False
        self.done = False
        self.next = None

    def main_start(self):
        """Called just before the mainloop starts
        Put your own stuff here.
        """
        pass

    def mainloop(self):
        """Mainloop
        Loop functions are called in this order:
        - check_events()
        - check_collisions()
        - update_screen()
        - update_other()
        - tick()

        returns next and paused
        """
        self.main_start()
        while not self.done:
            self.check_events()
            self.check_collisions()
            self.update_screen()
            self.update_other()
            self.tick()
            pygame.event.pump()
        return self.next, self.paused

    def check_events(self):
        """Checks for events, updates widgets
        """

        for ev in pygame.event.get():
            if ev.type == QUIT:
                self.close_game()
            for i in self.events:
                ev2 = i.ev
                #check keyboard events
                if ev.type==KEYUP or ev.type==KEYDOWN:
                    if ev.type == ev2.type and \
                        ev.key==ev2.key:
                        i.call()
                #check mouse events
                elif ev.type==MOUSEBUTTONDOWN or ev.type==MOUSEBUTTONUP:
                    if ev.type==ev2.type and \
                        ev.button==ev2.button:
                        i.call()

            #update widgets
            for w in self.widgets:
                w.update()

    def add_kbevent(self, type, key, callback, arg=None):
        """Add Keyboard event
        type: KEYUP or KEYDOWN
        key: pygame key constant (like K_r or K_SPACE)
        callback: function that should be called when key is pressed
        arg: argument for callback (optional)

        Example:
        self.events.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)
        (Meaning: if the player presses ESCAPE the game will quit)
        """
        ev = Event(type, {"key":key,"mod":0}, callback=callback, arg=arg)
        self.events.add(ev)

    def check_collisions(self):
        """Checks for sprite collisions
        Put your own collision stuff here
        """
        pass

    def update_screen(self):
        """Is supposed to update screen
        Nothing here, so you can handle screen updates any way you like.
        """
        pass

    def tick(self):
        """Caps framerate to 20
        """
        self.clock.tick(self.fps)

    def update_other(self):
        """Use that method to add your other stuff that
        needs to be updated every frame.
        """
        pass

    def quit(self):
        """Sets quit flag to True
        """
        self.done = True

    def close_game(self):
        """Closes the entire game window
        """
        sys.exit(0)
